@@ -1,5 +1,5 @@
 require_relative "testbase"
-require_relative "../ruby/envutil"
+require 'bigdecimal/math'
 
 require 'thread'
 
@@ -85,6 +85,9 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_raise(ArgumentError) { BigDecimal(0.1) }
     assert_raise(ArgumentError) { BigDecimal(0.1, Float::DIG + 2) }
     assert_nothing_raised { BigDecimal(0.1, Float::DIG + 1) }
+
+    bug9214 = '[ruby-core:58858]'
+    assert_equal(BigDecimal(-0.0, Float::DIG).sign, -1, bug9214)
   end
 
   def test_global_new_with_big_decimal
@@ -100,6 +103,13 @@ class TestBigDecimal < Test::Unit::TestCase
     end
   end
 
+  def test_global_new_with_tainted_string
+    Thread.new {
+      $SAFE = 1
+      BigDecimal('1'.taint)
+    }.join
+  end
+
   def test_new
     assert_equal(1, BigDecimal.new("1"))
     assert_equal(1, BigDecimal.new("1", 1))
@@ -150,6 +160,13 @@ class TestBigDecimal < Test::Unit::TestCase
     end
   end
 
+  def test_new_with_tainted_string
+    Thread.new {
+      $SAFE = 1
+      BigDecimal.new('1'.taint)
+    }.join
+  end
+
   def _test_mode(type)
     BigDecimal.mode(type, true)
     assert_raise(FloatDomainError) { yield }
@@ -420,6 +437,20 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(0, BigDecimal("1E-1") <=> 10**(-1), '#4825')
   end
 
+  def test_cmp_issue9192
+    bug9192 = '[ruby-core:58756] [#9192]'
+    operators = { :== => :==, :< => :>, :> => :<, :<= => :>=, :>= => :<= }
+    5.upto(8) do |i|
+      s = "706.0#{i}"
+      d = BigDecimal(s)
+      f = s.to_f
+      operators.each do |op, inv|
+        assert_equal(d.send(op, f), f.send(inv, d),
+                     "(BigDecimal(#{s.inspect}) #{op} #{s}) and (#{s} #{inv} BigDecimal(#{s.inspect})) is different #{bug9192}")
+      end
+    end
+  end
+
   def test_cmp_nan
     n1 = BigDecimal.new("1")
     BigDecimal.mode(BigDecimal::EXCEPTION_NaN, false)
@@ -590,8 +621,8 @@ class TestBigDecimal < Test::Unit::TestCase
 
   def test_coerce
     a, b = BigDecimal.new("1").coerce(1.0)
-    assert_instance_of(Float, a)
-    assert_instance_of(Float, b)
+    assert_instance_of(BigDecimal, a)
+    assert_instance_of(BigDecimal, b)
     assert_equal(2, 1 + BigDecimal.new("1"), '[ruby-core:25697]')
 
     a, b = BigDecimal("1").coerce(1.quo(10))
@@ -638,6 +669,14 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(BigDecimal.new((2**100-1).to_s), x - 1)
   end
 
+  def test_sub_with_float
+    assert_kind_of(BigDecimal, BigDecimal.new("3") - 1.0)
+  end
+
+  def test_sub_with_rational
+    assert_kind_of(BigDecimal, BigDecimal.new("3") - 1.quo(3))
+  end
+
   def test_mult
     x = BigDecimal.new((2**100).to_s)
     assert_equal(BigDecimal.new((2**100 * 3).to_s), (x * 3).to_i)
@@ -646,6 +685,20 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(BigDecimal.new((2**200).to_s), (x * x).to_i)
   end
 
+  def test_mult_with_float
+    assert_kind_of(BigDecimal, BigDecimal.new("3") * 1.5)
+  end
+
+  def test_mult_with_rational
+    assert_kind_of(BigDecimal, BigDecimal.new("3") * 1.quo(3))
+  end
+
+  def test_mult_with_nil
+    assert_raise(TypeError) {
+      BigDecimal('1.1') * nil
+    }
+  end
+
   def test_div
     x = BigDecimal.new((2**100).to_s)
     assert_equal(BigDecimal.new((2**100 / 3).to_s), (x / 3).to_i)
@@ -653,6 +706,18 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(BigDecimal::SIGN_NEGATIVE_ZERO, (BigDecimal.new("-0") / 1).sign)
     assert_equal(2, BigDecimal.new("2") / 1)
     assert_equal(-2, BigDecimal.new("2") / -1)
+
+    assert_equal(BigDecimal('1486.868686869'), BigDecimal('1472.0') / BigDecimal('0.99'), '[ruby-core:59365] [#9316]')
+
+    assert_equal(4.124045235, BigDecimal('0.9932') / (700 * BigDecimal('0.344045') / BigDecimal('1000.0')), '[#9305]')
+  end
+
+  def test_div_with_float
+    assert_kind_of(BigDecimal, BigDecimal.new("3") / 1.5)
+  end
+
+  def test_div_with_rational
+    assert_kind_of(BigDecimal, BigDecimal.new("3") / 1.quo(3))
   end
 
   def test_mod
@@ -663,6 +728,14 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(-1, (-x) % -3)
   end
 
+  def test_mod_with_float
+    assert_kind_of(BigDecimal, BigDecimal.new("3") % 1.5)
+  end
+
+  def test_mod_with_rational
+    assert_kind_of(BigDecimal, BigDecimal.new("3") % 1.quo(3))
+  end
+
   def test_remainder
     x = BigDecimal.new((2**100).to_s)
     assert_equal(1, x.remainder(3))
@@ -671,6 +744,14 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(-1, (-x).remainder(-3))
   end
 
+  def test_remainder_with_float
+    assert_kind_of(BigDecimal, BigDecimal.new("3").remainder(1.5))
+  end
+
+  def test_remainder_with_rational
+    assert_kind_of(BigDecimal, BigDecimal.new("3").remainder(1.quo(3)))
+  end
+
   def test_divmod
     x = BigDecimal.new((2**100).to_s)
     assert_equal([(x / 3).floor, 1], x.divmod(3))
@@ -749,6 +830,20 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(1, BigDecimal.new("1").sqrt(1))
   end
 
+  def test_sqrt_5266
+    x = BigDecimal('2' + '0'*100)
+    assert_equal('0.14142135623730950488016887242096980785696718753769480731',
+                 x.sqrt(56).to_s(56).split(' ')[0])
+    assert_equal('0.1414213562373095048801688724209698078569671875376948073',
+                 x.sqrt(55).to_s(55).split(' ')[0])
+
+    x = BigDecimal('2' + '0'*200)
+    assert_equal('0.14142135623730950488016887242096980785696718753769480731766797379907324784621070388503875343276415727350138462',
+                 x.sqrt(110).to_s(110).split(' ')[0])
+    assert_equal('0.1414213562373095048801688724209698078569671875376948073176679737990732478462107038850387534327641572735013846',
+                 x.sqrt(109).to_s(109).split(' ')[0])
+  end
+
   def test_fix
     x = BigDecimal.new("1.1")
     assert_equal(1, x.fix)
@@ -860,6 +955,12 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_match(/^#<BigDecimal:[0-9a-f]+,'0.12345678E4',#{prec}\(#{maxprec}\)>$/, x.inspect)
   end
 
+  def test_power
+    assert_nothing_raised(TypeError, '[ruby-core:47632]') do
+      1000.times { BigDecimal.new('1001.10')**0.75 }
+    end
+  end
+
   def test_power_with_nil
     assert_raise(TypeError) do
       BigDecimal(3) ** nil
@@ -1048,6 +1149,9 @@ class TestBigDecimal < Test::Unit::TestCase
     e   = BigDecimal("2.71828182845904523536028747135266249775724709369996")
     pow = BigDecimal("22.459157718361045473")
     assert_equal(pow, pi.power(e, 20))
+
+    b = BigDecimal('1.034482758620689655172413793103448275862068965517241379310344827586206896551724')
+    assert_equal(BigDecimal('0.114523E1'), b.power(4, 5), '[Bug #8818] [ruby-core:56802]')
   end
 
   def test_limit
@@ -1056,6 +1160,25 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_equal(90, x ** 4) # OK? must it be 80?
     # 3 * 3 * 3 * 3 = 10 * 3 * 3 = 30 * 3 = 90 ???
     assert_raise(ArgumentError) { BigDecimal.limit(-1) }
+
+    bug7458 = '[ruby-core:50269] [#7458]'
+    one = BigDecimal('1')
+    epsilon = BigDecimal('0.7E-18')
+    BigDecimal.save_limit do
+      BigDecimal.limit(0)
+      assert_equal(BigDecimal("1.0000000000000000007"), one + epsilon, "limit(0) #{bug7458}")
+
+      1.upto(18) do |lim|
+        BigDecimal.limit(lim)
+        assert_equal(BigDecimal("1.0"), one + epsilon, "limit(#{lim}) #{bug7458}")
+      end
+
+      BigDecimal.limit(19)
+      assert_equal(BigDecimal("1.000000000000000001"), one + epsilon, "limit(19) #{bug7458}")
+
+      BigDecimal.limit(20)
+      assert_equal(BigDecimal("1.0000000000000000007"), one + epsilon, "limit(20) #{bug7458}")
+    end
   end
 
   def test_sign
@@ -1216,11 +1339,35 @@ class TestBigDecimal < Test::Unit::TestCase
   end
 
   def test_BigMath_exp
-    n = 20
-    assert_in_epsilon(Math.exp(n), BigMath.exp(BigDecimal("20"), n))
-    assert_in_epsilon(Math.exp(40), BigMath.exp(BigDecimal("40"), n))
-    assert_in_epsilon(Math.exp(-n), BigMath.exp(BigDecimal("-20"), n))
-    assert_in_epsilon(Math.exp(-40), BigMath.exp(BigDecimal("-40"), n))
+    prec = 20
+    assert_in_epsilon(Math.exp(20), BigMath.exp(BigDecimal("20"), prec))
+    assert_in_epsilon(Math.exp(40), BigMath.exp(BigDecimal("40"), prec))
+    assert_in_epsilon(Math.exp(-20), BigMath.exp(BigDecimal("-20"), prec))
+    assert_in_epsilon(Math.exp(-40), BigMath.exp(BigDecimal("-40"), prec))
+  end
+
+  def test_BigMath_exp_with_float
+    prec = 20
+    assert_in_epsilon(Math.exp(20), BigMath.exp(20.0, prec))
+    assert_in_epsilon(Math.exp(40), BigMath.exp(40.0, prec))
+    assert_in_epsilon(Math.exp(-20), BigMath.exp(-20.0, prec))
+    assert_in_epsilon(Math.exp(-40), BigMath.exp(-40.0, prec))
+  end
+
+  def test_BigMath_exp_with_fixnum
+    prec = 20
+    assert_in_epsilon(Math.exp(20), BigMath.exp(20, prec))
+    assert_in_epsilon(Math.exp(40), BigMath.exp(40, prec))
+    assert_in_epsilon(Math.exp(-20), BigMath.exp(-20, prec))
+    assert_in_epsilon(Math.exp(-40), BigMath.exp(-40, prec))
+  end
+
+  def test_BigMath_exp_with_rational
+    prec = 20
+    assert_in_epsilon(Math.exp(20), BigMath.exp(Rational(40,2), prec))
+    assert_in_epsilon(Math.exp(40), BigMath.exp(Rational(80,2), prec))
+    assert_in_epsilon(Math.exp(-20), BigMath.exp(Rational(-40,2), prec))
+    assert_in_epsilon(Math.exp(-40), BigMath.exp(Rational(-80,2), prec))
   end
 
   def test_BigMath_exp_under_gc_stress
@@ -1237,12 +1384,24 @@ class TestBigDecimal < Test::Unit::TestCase
     EOS
   end
 
+  def test_BigMath_log_with_string
+    assert_raise(ArgumentError) do
+      BigMath.log("foo", 20)
+    end
+  end
+
   def test_BigMath_log_with_nil
     assert_raise(ArgumentError) do
       BigMath.log(nil, 20)
     end
   end
 
+  def test_BigMath_log_with_non_integer_precision
+    assert_raise(ArgumentError) do
+      BigMath.log(1, 0.5)
+    end
+  end
+
   def test_BigMath_log_with_nil_precision
     assert_raise(ArgumentError) do
       BigMath.log(1, nil)
@@ -1255,7 +1414,19 @@ class TestBigDecimal < Test::Unit::TestCase
     end
   end
 
-  def test_BigMath_log_with_zerp_precision
+  def test_BigMath_log_with_zero_arg
+    assert_raise(Math::DomainError) do
+      BigMath.log(0, 20)
+    end
+  end
+
+  def test_BigMath_log_with_negative_arg
+    assert_raise(Math::DomainError) do
+      BigMath.log(-1, 20)
+    end
+  end
+
+  def test_BigMath_log_with_zero_precision
     assert_raise(ArgumentError) do
       BigMath.log(1, 0)
     end
@@ -1291,6 +1462,13 @@ class TestBigDecimal < Test::Unit::TestCase
     end
   end
 
+  def test_BigMath_log_with_float_nan
+    BigDecimal.save_exception_mode do
+      BigDecimal.mode(BigDecimal::EXCEPTION_NaN, false)
+      assert(BigMath.log(Float::NAN, 20).nan?)
+    end
+  end
+
   def test_BigMath_log_with_1
     assert_in_delta(0.0, BigMath.log(1, 20))
     assert_in_delta(0.0, BigMath.log(1.0, 20))
@@ -1298,7 +1476,7 @@ class TestBigDecimal < Test::Unit::TestCase
   end
 
   def test_BigMath_log_with_exp_1
-    assert_in_delta(1.0, BigMath.log(BigMath.exp(1, 20), 20))
+    assert_in_delta(1.0, BigMath.log(BigMath.E(10), 10))
   end
 
   def test_BigMath_log_with_2
@@ -1307,8 +1485,15 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_in_delta(Math.log(2), BigMath.log(BigDecimal(2), 20))
   end
 
-  def test_BigMath_log_with_square_of_exp_2
-    assert_in_delta(2, BigMath.log(BigMath.exp(1, 20)**2, 20))
+  def test_BigMath_log_with_square_of_E
+    assert_in_delta(2, BigMath.log(BigMath.E(20)**2, 20))
+  end
+
+  def test_BigMath_log_with_high_precision_case
+    e   = BigDecimal('2.71828182845904523536028747135266249775724709369996')
+    e_3 = e.mult(e, 50).mult(e, 50)
+    log_3 = BigMath.log(e_3, 50)
+    assert_in_delta(3, log_3, 0.0000000000_0000000000_0000000000_0000000000_0000000001)
   end
 
   def test_BigMath_log_with_42
@@ -1317,6 +1502,11 @@ class TestBigDecimal < Test::Unit::TestCase
     assert_in_delta(Math.log(42), BigMath.log(BigDecimal(42), 20))
   end
 
+  def test_BigMath_log_with_101
+    # this is mainly a performance test (should be very fast, not the 0.3 s)
+    assert_in_delta(Math.log(101), BigMath.log(101, 20), 1E-15)
+  end
+
   def test_BigMath_log_with_reciprocal_of_42
     assert_in_delta(Math.log(1e-42), BigMath.log(1e-42, 20))
     assert_in_delta(Math.log(1e-42), BigMath.log(BigDecimal("1e-42"), 20))
@@ -1362,4 +1552,31 @@ class TestBigDecimal < Test::Unit::TestCase
     Thread.current.keys.to_s
     EOS
   end
+
+  def assert_no_memory_leak(code, *rest, **opt)
+    code = "8.times {20_000.times {begin #{code}; rescue NoMemoryError; end}; GC.start}"
+    super(["-rbigdecimal"],
+          "b = BigDecimal('10'); b.nil?; " \
+          "GC.add_stress_to_class(BigDecimal); "\
+          "#{code}", code, *rest, rss: true, limit: 1.1, **opt)
+  end
+
+  if EnvUtil.gc_stress_to_class?
+    def test_no_memory_leak_allocate
+      assert_no_memory_leak("BigDecimal.allocate")
+    end
+
+    def test_no_memory_leak_initialize
+      assert_no_memory_leak("BigDecimal.new")
+    end
+
+    def test_no_memory_leak_global_new
+      assert_no_memory_leak("BigDecimal('10')")
+      assert_no_memory_leak("BigDecimal(b)")
+    end
+
+    def test_no_memory_leak_create
+      assert_no_memory_leak("b + 10")
+    end
+  end
 end
